from os import path
from setuptools import setup


with open(path.join(path.abspath(path.dirname(__file__)), 'README.md')) as f:
    long_description = f.read()


setup(
    name='bolt11',
    version='0.0.0',
    url='https://github.com/python-ln/bolt11',
    author='eillarra',
    author_email='eneko@illarra.com',
    license='MIT',
    description='A library for encoding and decoding BOLT11 payment requests.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='bitcoin lightning-network bolt11',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
    packages=['bolt11'],
    install_requires=[
    ],
    zip_safe=False
)



import re

from .types import Bitcoin


def _btc_to_amount(btc: Bitcoin) -> str:
    """ Given an amount in bitcoin, shorten it.
    """
    btc = int(btc * 10 ** 12)

    for unit in ['p', 'n', 'u', 'm', '']:
        if btc % 1000 == 0:
            btc //= 1000
        else:
            break

    return f'{btc}{unit}'


def _amount_to_btc(amount: str) -> Bitcoin:
    """ Given a shortened amount, convert it into bitcoin.
    """
    if not re.fullmatch(r'\d+[pnum]?', amount):
        raise ValueError(f'Invalid amount "{amount}"')

    try:
        num = {
            'p': 10 ** 12,
            'n': 10 ** 9,
            'u': 10 ** 6,
            'm': 10 ** 3,
        }[amount[-1]]
        return Bitcoin(amount[:-1]) / num

    except KeyError:
        return Bitcoin(amount)



from decimal import Decimal


class Bitcoin(Decimal):

    @property
    def sats(self) -> int:
        return (self * 10 ** 8).to_integral_exact()


class Bech32(str):
    __slots__ = ('hrp', 'data')


class LightningInvoice(Bech32):
    pass
